from symbl.utils.Helper import dictionary_to_valid_json, correct_boolean_values, initialize_api_client
from symbl.Conversations import Conversation
from symbl_rest import AsyncApi as async_api_rest
from symbl.utils.Logger import Log

class Video():

    def __init__(self):
        '''
            It will initialize the Analysis class
            with the object of Initialize Class
        '''
        self.__async_api_rest = async_api_rest()

    @initialize_api_client
    def process_file(self, file_path:str, credentials=None, content_type:str='video/mp4', wait:bool=True, parameters={}):
        '''
            video files to be analyzed
            returns Conversation object
        '''
        if file_path == None:
            raise ValueError("Please enter a valid file_path")
        
        params = dictionary_to_valid_json(parameters)

        file = open(file_path, 'rb')
        video_file = file.read()
        response = self.__async_api_rest.add_video(body=video_file, content_type=content_type, **correct_boolean_values(params))
        Log.getInstance().info("Job with jobId {} for conversationId {} started".format(response.job_id, response.conversation_id))

        return Conversation(response.conversation_id, response.job_id, wait=wait, credentials=credentials)


    @initialize_api_client
    def process_url(self, payload:dict, credentials=None, wait:bool=True, parameters={}):
        '''
            url of audio file to be analyzed
            returns Conversation object
        '''

        if 'url' not in payload or payload['url'] == None:
            raise ValueError("Please enter a valid url.")
        
        params = dictionary_to_valid_json(parameters)

        response = self.__async_api_rest.add_video_url(body=payload, **correct_boolean_values(params))
        Log.getInstance().info("Job with jobId {} for conversationId {} started".format(response.job_id, response.conversation_id))

        return Conversation(response.conversation_id, response.job_id, wait=wait, credentials=credentials)


    @initialize_api_client
    def append_file(self, file_path:str, conversation_id:str, credentials=None, content_type:str='video/mp4', wait:bool=True, parameters={}):
        '''
            video files to be appended
            returns Conversation object
        '''
        if file_path == None:
            raise ValueError("Please enter a valid file_path")

        if conversation_id == None or len(conversation_id) == 0:
            raise ValueError("Please enter a valid conversation_id")
        
        params = dictionary_to_valid_json(parameters)

        file = open(file_path, 'rb')
        video_file = file.read()
        response = self.__async_api_rest.append_video(body=video_file, content_type=content_type, conversation_id=conversation_id, **correct_boolean_values(params))
        Log.getInstance().info("Job with jobId {} for conversationId {} started".format(response.job_id, response.conversation_id))

        return Conversation(response.conversation_id, response.job_id, wait=wait, credentials=credentials)


    @initialize_api_client
    def append_url(self, payload:dict, conversation_id:str, credentials=None, wait:bool=True, parameters={}):
        '''
            url of video file to be appended
            returns Conversation object
        '''
        if 'url' not in payload or payload['url'] == None:
            raise ValueError("Please enter a valid url")

        if conversation_id == None or len(conversation_id) == 0:
            raise ValueError("Please enter a valid conversation_id")
        
        params = dictionary_to_valid_json(parameters)

        response = self.__async_api_rest.append_video_url(body=payload, conversation_id=conversation_id, **correct_boolean_values(params))
        Log.getInstance().info("Job with jobId {} for conversationId {} started".format(response.job_id, response.conversation_id))

        return Conversation(response.conversation_id, response.job_id, wait=wait, credentials=credentials)


from symbl.utils.Helper import correct_boolean_values, dictionary_to_valid_json, initialize_api_client, insert_valid_boolean_values
from symbl_rest import ConversationsApi as conversations_api_rest
from symbl.utils import Helper

class ConversationsApi():

    def __init__(self):
        '''
            It will initialize the ConversationsApi class
        '''
        
        self.conversations_api_rest = conversations_api_rest()

    @initialize_api_client
    def get_action_items(self, conversation_id, credentials=None):
        try:
            return self.conversations_api_rest.get_action_items_by_conversation_id(conversation_id)
        except:
            return {'message': 'The conversationId is either invalid or does not exist.'}


    @initialize_api_client
    def get_call_score(self, conversation_id, credentials=None):
        try:
            return self.conversations_api_rest.get_call_score_by_conversation_id(conversation_id)
        except:
            return {'message': 'The conversationId is either invalid or does not exist.'}

    @initialize_api_client
    def get_call_score_insights_url(self, conversation_id, credentials=None):
        try:
            params = {'include_call_score': True}
            return self.conversations_api_rest.get_experience_url_by_conversation_id(conversation_id, **correct_boolean_values(params))
        except Exception as e:
            return {'message': 'The conversationId is either invalid or does not exist.'}

    @initialize_api_client
    def get_call_score_status(self, conversation_id, credentials=None):
        try:
            res = self.conversations_api_rest.get_call_score_status_by_conversation_id(conversation_id)
            return res
        except Exception as e:
            return {'message': 'The conversationId is either invalid or does not exist.'}

    @initialize_api_client
    def get_follow_ups(self, conversation_id, credentials=None ):  
        try:
            return self.conversations_api_rest.get_follow_ups_by_conversation_id(conversation_id)
        except:
            return {'message': 'The conversationId is either invalid or does not exist.'}
  
    @initialize_api_client      
    def get_members(self, conversation_id, credentials=None):  
        try:  
            return self.conversations_api_rest.get_members_by_conversation_id(conversation_id)
        except:
            return {'message': 'The conversationId is either invalid or does not exist.'}
  
  
    @initialize_api_client      
    def get_messages(self, conversation_id, credentials=None, parameters={}):  
        params = dictionary_to_valid_json(parameters)
        try:
            return self.conversations_api_rest.get_messages_by_conversation_id(conversation_id, **correct_boolean_values(params))
        except:
            return {'message': 'The conversationId is either invalid or does not exist.'}
  
    @initialize_api_client      
    def get_questions(self, conversation_id, credentials=None):  
        try:
            return self.conversations_api_rest.get_questions_by_conversation_id(conversation_id)
        except:
            return {'message': 'The conversationId is either invalid or does not exist.'}
  
  
    @initialize_api_client      
    def get_topics(self, conversation_id, credentials=None, parameters={}):
        try:
            params = dictionary_to_valid_json(parameters)
            return self.conversations_api_rest.get_topics_by_conversation_id(conversation_id, **correct_boolean_values(params))
        except:
            return {'message': 'The conversationId is either invalid or does not exist.'}
  

    @initialize_api_client      
    def get_conversation(self, conversation_id, credentials=None):
        try:
          return self.conversations_api_rest.get_conversation_by_conversation_id(conversation_id)
        except:
            return {'message': 'The conversationId is either invalid or does not exist.'}
  

    @initialize_api_client      
    def get_trackers(self, conversation_id, credentials=None):
        try:
          return self.conversations_api_rest.get_trackers_by_conversation_id(conversation_id)
        except:
            return {'message': 'The conversationId is either invalid or does not exist.'}
  

    @initialize_api_client      
    def get_entities(self, conversation_id, credentials=None):
        try:
            api_response = self.conversations_api_rest.get_entities_by_conversation_id(conversation_id)
            return Helper.parse_entity_response(api_response) if len(api_response.entities)!=0 else api_response
        except:
            return {'message': 'The conversationId is either invalid or does not exist.'}
  

    @initialize_api_client      
    def get_analytics(self, conversation_id, credentials=None):
        try:
            return self.conversations_api_rest.get_analytics_by_conversation_id(conversation_id)
        except:
            return {'message': 'The conversationId is either invalid or does not exist.'}
  

    @initialize_api_client
    def put_members(self, conversation_id, members_id, parameters={}, credentials=None):
        try:
            return self.conversations_api_rest.put_members_information_by_members_id(conversation_id, members_id, body=parameters)
        except:
            return {'message': 'The conversationId is either invalid or does not exist.'}
  

    @initialize_api_client
    def put_speakers_events(self, conversation_id, parameters={}, credentials=None):
        try:
          return self.conversations_api_rest.put_speakers_event_by_conversation_id(conversation_id, body=parameters)
        except:
            return {'message': 'The conversationId is either invalid or does not exist.'}
  

    @initialize_api_client      
    def delete_conversation(self, conversation_id, credentials=None):
        try:
            return self.conversations_api_rest.delete_conversation_by_conversation_id(conversation_id)
        except:
            pass

    @initialize_api_client      
    def get_formatted_transcript(self, conversation_id, parameters = {},credentials=None):
        try:
            content_type = "application/json"
            params = insert_valid_boolean_values(parameters)
            return self.conversations_api_rest.get_formatted_transcript_by_conversation_id(params,content_type,conversation_id)
        except:
            return {'message': 'The conversationId is either invalid or does not exist.'}
  


from symbl_rest import AuthenticationApi, Configuration, ApiClient

import configparser
import threading
import os

SYMBL_CONFIGURATION_FILENAME = 'symbl.conf'
SYMBL_CREDENTIALS_SECTION = 'credentials'
SYMBL_APP_ID_CONSTANT = 'app_id'
SYMBL_APP_SECRET_CONSTANT = 'app_secret'

configuration = None

cached_app_id = ""
cached_app_secret = ""


config_parser = configparser.ConfigParser()


'''
    It will return an object of ApiClient with a prefilled token. 
'''

def init(app_id=None, app_secret=None):
    if app_id == None and app_secret == None:
        app_id, app_secret = fetchAppCredentialsFromFiles()
    
    generateAndInitializeToken(app_id=app_id, app_secret=app_secret)
    

def generateAndInitializeToken(app_id, app_secret):

    if app_id is None or len(app_id) == 0:
        raise ValueError('app_id is required')
    
    if app_secret is None or len(app_secret) == 0:
        raise ValueError('app_secret is required')
        
    generate_token(app_id, app_secret)

def fetchAppCredentialsFromFiles():
    # configuration file exists in local directory
    if os.path.isfile(SYMBL_CONFIGURATION_FILENAME):
        config_parser.read_file(open(SYMBL_CONFIGURATION_FILENAME))
        if config_parser.has_option(SYMBL_CREDENTIALS_SECTION, SYMBL_APP_ID_CONSTANT) and config_parser.has_option(SYMBL_CREDENTIALS_SECTION, SYMBL_APP_SECRET_CONSTANT):
            return config_parser.get(SYMBL_CREDENTIALS_SECTION, SYMBL_APP_ID_CONSTANT), config_parser.get(SYMBL_CREDENTIALS_SECTION, SYMBL_APP_SECRET_CONSTANT)
        else:
            # configuration file exists in home directory of system
            return fetchAppCredentialsInPath()
    else:
        # configuration file exists in home directory of system
        return fetchAppCredentialsInPath()

def fetchAppCredentialsInPath():
    app_id, app_secret = None, None
    if os.path.isfile("{}/{}".format(os.path.expanduser('~'), SYMBL_CONFIGURATION_FILENAME)):
        config_parser.read_file(open("{}/{}".format(os.path.expanduser('~'), SYMBL_CONFIGURATION_FILENAME)))
        if config_parser.has_option(SYMBL_CREDENTIALS_SECTION, SYMBL_APP_ID_CONSTANT) and config_parser.has_option(SYMBL_CREDENTIALS_SECTION, SYMBL_APP_SECRET_CONSTANT):
            app_id, app_secret =  config_parser.get(SYMBL_CREDENTIALS_SECTION, SYMBL_APP_ID_CONSTANT), config_parser.get(SYMBL_CREDENTIALS_SECTION, SYMBL_APP_SECRET_CONSTANT)

    return app_id, app_secret


def generate_token(app_id, app_secret):
    configuration_instance = Configuration()
    body={
        'type': 'application', 
        'appId': app_id, 
        'appSecret': app_secret
        }

    authenticationResponse = AuthenticationApi().generate_token(body)
    # refreshing token 60 secs before current one expires
    time = authenticationResponse.expires_in

    global configuration
    
    timer = threading.Timer(time, generate_token, args=[app_id, app_secret])
    timer.daemon = True
    timer.start()

    configuration_instance.api_key['x-api-key'] = authenticationResponse.access_token
    configuration = configuration_instance

def get_api_client(credentials=None):

    app_id, app_secret = None, None

    if credentials != None and 'app_id' in credentials:
        app_id = credentials['app_id']
    
    if credentials != None and 'app_secret' in credentials:
        app_secret = credentials['app_secret']

    global configuration
    if configuration == None:
        init(app_id, app_secret)
    return ApiClient(configuration)

def get_api_header(credentials=None):
    app_id, app_secret = None, None

    if credentials != None and 'app_id' in credentials:
        app_id = credentials['app_id']
    
    if credentials != None and 'app_secret' in credentials:
        app_secret = credentials['app_secret']

    global configuration
    if configuration == None:
        init(app_id, app_secret)

    return configuration.api_key

def get_access_token(credentials=None):
    app_id, app_secret = None, None

    if credentials != None and 'app_id' in credentials:
        app_id = credentials['app_id']
    
    if credentials != None and 'app_secret' in credentials:
        app_secret = credentials['app_secret']

    global configuration
    if configuration == None:
        init(app_id, app_secret)

    return configuration.api_key['x-api-key']


def isEmpty(value):
    return value == None or len(value) == 0

def validateActions(actions):
    invalidActions = []
    for action in actions:
        if isEmpty(action["invokeOn"]) and isEmpty(action["name"]):
            invalidActions.append(action)
        
    if len(invalidActions) > 0 :
        raise ValueError({'message': "Invalid actions detected. Count: ${invalidActions.length}", "actions": invalidActions})

def validateEndpoint(endpoint):
    if isEmpty(endpoint["type"]):
        raise ValueError("endpoint type is required.")
    
    if endpoint["type"].lower() == "pstn":
        if isEmpty(endpoint["phoneNumber"]):
            raise ValueError("phoneNumber is required when type = 'pstn'.")
    
    elif endpoint["type"].lower() == "sip":
        if isEmpty(endpoint["uri"]):
            raise ValueError("uri is required when type = 'sip'.")
    
    else:
        raise ValueError("endpoint.type = '${endpoint.type}' is not valid. Supported types are ['pstn' , 'sip']")


import json
from symbl import AuthenticationToken
import datetime

DATE_FORMAT = '%Y-%m-%d %H:%M:%S'

def correct_boolean_values(dictionary: dict):
    for key in dictionary:
        if dictionary[key] == True and type(dictionary[key]) == bool:
            dictionary[key] = "true"
        elif dictionary[key] == False and type(dictionary[key]) == bool:
            dictionary[key] = "false"
    return dictionary

def insert_valid_boolean_values(dictionary: dict):
    for key in dictionary:
        if type(dictionary[key]) == dict:
            for dict_key in dictionary[key]:
                if dictionary[key][dict_key] == "true":
                    dictionary[key][dict_key] = True
                elif dictionary[key][dict_key] == "false":
                    dictionary[key][dict_key] = False
        elif dictionary[key] == "true":
            dictionary[key] = True
        elif dictionary[key] == "false":
            dictionary[key] = False
    return dictionary
    
def dictionary_to_valid_json(dictionary: dict):
    new_dictionary = dict()
    for key in dictionary.keys():
        new_key = ''.join(['_'+i.lower() if i.isupper() else i for i in key]).lstrip('_')
        if type(dictionary[key]) == list or type(dictionary[key]) == dict:
            new_dictionary[new_key] = json.dumps(dictionary[key])
        else:
            new_dictionary[new_key] = dictionary[key]
    
    return new_dictionary

def initialize_api_client(function):
    def wrapper(*args, **kw):
        credentials = None
        
        if 'credentials' in kw:
            credentials = kw['credentials']

        AuthenticationToken.get_api_client(credentials)

        return function(*args, **kw)
    
    return wrapper

#verify date format
def verify_date(date):
    try:
        datetime.datetime.strptime(date, '%Y-%m-%d')
        return True
    except ValueError:
        return False

#Parse date format
def deserialize_date(strISODateString):
    """Deserializes string to date.
    :param strISODateString: str.
    :return: date.
    """
    try:
        from datetime import datetime
        return datetime.strptime(strISODateString, DATE_FORMAT)
    except ImportError:
        return strISODateString
    except ValueError:
        raise Exception(
            status=0,
            reason="Failed to parse `{0}` as date object".format(strISODateString)
        )

#This function will remove the None values from the API response, and will also parse the date format
def parse_entity_response(api_response):
    api_response=api_response.entities
    count = len(api_response)
    entity_response=[]

    keys = ['custom_type','end','message_refs','start','type','text','value']

    for obj in range(0,count):
        entity_res = dict()
        for key in keys:
            val = getattr(api_response[obj],key)
            if val is not None:
                if key=='value': 
                    if verify_date(val):
                        val = "".join([val," 00:00:00"])
                        val = deserialize_date(val)
                entity_res[key]=val
        entity_response.append(entity_res) 

    return dict(entities=entity_response)

# Using copy because we support Python 3.4
def merge_two_dicts(x, y):
    z = x.copy()
    z.update(y) 
    return z  



from symbl.utils import wrap_keyboard_interrupt, Thread, Log
from symbl.utils.Helper import initialize_api_client
from symbl.jobs_api.JobStatus import JobStatus
from symbl_rest import JobsApi

import time

class Job():

    __INTERVAL_TIME_IN_SECONDS = 5  ## in seconds

    def __init__(self, job_id: str, conversation_id:str, wait=True):

        self.__job_id = job_id
        self.__conversation_id = conversation_id
        self.__success_func = None
        self.__jobs_api = JobsApi()
        self.__error_func = None
        self.__job_status = JobStatus.IN_PROGRESS
        self.__wait = wait

    def getConversationId(self):
        return self.__conversation_id

    def get_job_status(self):
        return self.__job_status.value

    def get_job_id(self):
        return self.__job_id

    def on_complete(self, func):
        self.__success_func = func
        return self

    def on_error(self, func):
        self.__error_func = func
        return self

    @initialize_api_client
    def __fetch_current_job_status(self, credentials=None):
        if self.__jobs_api is not None:
            response = self.__jobs_api.get_job_status(self.__job_id)
            self.__job_status = JobStatus(response.status)
        else:
            raise ValueError("Job object not initialized correctly. Please contact administrator.")

    def synchronous_monitor_job(self, conversation, interval=None, wait=True, credentials=None):
        if interval is None:
            interval = self.__INTERVAL_TIME_IN_SECONDS

        while self.__job_status != JobStatus.COMPLETED and self.__job_status != JobStatus.FAILED:
            time.sleep(interval)
            self.__fetch_current_job_status(credentials=credentials)
            Log.getInstance().info("Fetching latest status of job {0}, current status is {1}".format(self.__job_id, self.__job_status.value))

        if self.__job_status == JobStatus.COMPLETED and self.__success_func != None:
            self.__success_func(conversation)

        elif self.__error_func != None:
            self.__error_func(conversation)

    @wrap_keyboard_interrupt
    def monitor_job(self, conversation, interval=None, wait=True, credentials=None):
        if wait:
            self.synchronous_monitor_job(conversation, interval, wait, credentials)
        else:
            Thread.getInstance().start_on_thread(self.synchronous_monitor_job, conversation, interval, wait, credentials)



from symbl.Conversations import Conversation

import unittest
from unittest.mock import Mock, patch

from types import SimpleNamespace


class ConversationsTest(unittest.TestCase):

    def test_get_messages_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(**{"messages":[{"id":"6125275260649472","text":"Hey, is this a question?","from":{},"startTime":"2021-05-27T13:19:25.270Z","endTime":"2021-05-27T13:19:26.770Z","conversationId":"6549352073920512","phrases":[]}]})
        conversations_object = Conversation("6549352073920512")

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.get_messages", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.get_messages())

    def test_get_action_items_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(**{"actionItems":[{"id":"4755377456414720","text":"Steve needs to complete the analysis by next Monday.","type":"action_item","score":0.9765610554943914,"messageIds":["5590459222065152"],"entities":[{"type":"daterange","text":"by next monday","offset":39,"end":"2021-06-07"},{"type":"person","text":"Steve","offset":0,"value":{"assignee":True,"id":"98d11b11-4a0a-4bdd-b2e1-d50238f605d6","name":"Steve","userId":"Steve@example.com"}}],"phrases":[],"from":{"id":"98d11b11-4a0a-4bdd-b2e1-d50238f605d6","name":"Steve","userId":"Steve@example.com"},"definitive":True,"assignee":{"id":"98d11b11-4a0a-4bdd-b2e1-d50238f605d6","name":"Steve","email":"Steve@example.com"}}]})
        conversations_object = Conversation("6549352073920512")

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.get_action_items", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.get_action_items())

    def test_get_follow_ups_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(**{"followUps":[{"id":"4529863081852928","text":"Steve needs to follow up about this next Monday.","type":"follow_up","score":1,"messageIds":["5399026486738944"],"entities":[{"type":"date","text":"next monday","offset":38,"value":"2021-06-07"},{"type":"person","text":"Steve","offset":0,"value":{"assignee":True,"id":"e2219a6c-ec5e-4412-94d7-46f565ba5eb7","name":"Steve","userId":"Steve@example.com"}}],"phrases":[],"from":{"id":"e2219a6c-ec5e-4412-94d7-46f565ba5eb7","name":"Steve","userId":"Steve@example.com"},"definitive":True,"assignee":{"id":"e2219a6c-ec5e-4412-94d7-46f565ba5eb7","name":"Steve","email":"Steve@example.com"}}]})
        conversations_object = Conversation("6549352073920512")

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.get_follow_ups", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.get_follow_ups())

    def test_get_members_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(**{"members":[{"id":"25f497bd-2800-4e2b-bc71-e143a8f71d6b","name":"Steve","email":"steve@abccorp.com"}]})
        conversations_object = Conversation("6549352073920512")

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.get_members", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.get_members())

    def test_get_questions_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(**{"questions":[{"id":"4784246146203648","text":"Will you be reviewint the analysis by next Monday?","type":"question","score":0.9833490590134427,"messageIds":["4580678118146048"],"from":{"id":"ca0fcd23-dc63-4d42-894a-43ca84650f6f","name":"Steve","userId":"Steve@example.com"}}]})
        conversations_object = Conversation("6549352073920512")

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.get_questions", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.get_questions())

    def test_get_topics_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(**{"topics":[{"id":"5506508885327872","text":"video","type":"topic","score":0.064,"messageIds":["5823121492803584","4673825691140096"],"parentRefs":[]}]})
        conversations_object = Conversation("6549352073920512")

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.get_topics", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.get_topics())

    def test_get_conversation_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(**{"id": "5102778889273344","type": "meeting","name": "TestingTextAPI","startTime": "2021-07-27T07:11:04.304Z","endTime": "2021-07-27T07:13:19.184Z","members": [{"id": "461dd687-4341-48a5-9f3a-7f6019d6378c","name": "John","email": "john@example.com"}]})
        conversations_object = Conversation("6549352073920512")

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.get_conversation", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.get_conversation())

    def test_get_trackers_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(**{ "id": "5243525907087360", "name": "text_tracker", "matches": [ { "messageRefs": [ { "id": "5867975128121344", "text": "I don't know which platform is it, but I came to know that platform.", "offset": 19 }, { "id": "6328818106105856", "text": "So this is a live demo that we are trying to give very we are going to show how the platform detects various insights can do transcription in real time and also the different topics of discussions, which would be generated after the call is over, and they will be an email that will be sent to the inbox.", "offset": 84 } ], "type": "vocabulary", "value": "platform", "insightRefs": [] } ] })
        conversations_object = Conversation("6549352073920512")

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.get_trackers", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.get_trackers())

    def test_get_entities_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(**{"entities":[{"custom_type":"None","type":"person","value":"richard holmes","text":"richard holmes","end":"None","start":"None","messageRefs":[{"id":"5895830172073984","text":"We need to have the meeting today, and we're going to talk about how to run a product strategy Workshop is by Richard Holmes.","offset":111}]}]})
        conversations_object = Conversation("6549352073920512")

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.get_entities", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.get_entities())

    def test_get_analytics_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(**{"metrics": [{"type": "total_silence","percent": 0,"seconds": 0},{"type": "total_talk_time","percent": 100,"seconds": 134.88},{"type": "total_overlap","percent": 0,"seconds": 0}],"members": [{"id": "461dd687-4341-48a5-9f3a-7f6019d6378c","name": "John","userId": "john@example.com","pace": {"wpm": 124},"talkTime": {"percentage": 100,"seconds": 134.88},"listenTime": {"percentage": 0,"seconds": 0},"overlap": {}}]})
        conversations_object = Conversation("6549352073920512")

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.get_analytics", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.get_analytics())

    def test_put_members_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(
            **{"message": "Member with id: 1234567890 for conversationId: 6549352073920512 updated successfully! The update should be reflected in all messages and insights along with this conversation"})
        conversations_object = Conversation("6549352073920512")

        param = {"id": "1234567890",
                 "email": "john@example.in",
                 "name": "john"}

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.put_members", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.put_members("1234567890", param))

    def test_put_speakers_events_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(
            **{"message": "Speaker events associated for conversationId: 6549352073920512 successfully! The update should be reflected in all messages and insights along with this conversation"})
        conversations_object = Conversation("6549352073920512")

        payload_streaming = {
            "speakerEvents": [
                {
                    "type": "started_speaking",
                    "user": {
                        "id": "1234567890",
                        "name": "john",
                        "email": "john@example.com"
                    },
                    "offset": {
                        "seconds": 52,
                        "nanos": 5000000000
                    }
                }
            ]
        }

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.put_speakers_events", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.put_speakers_events( payload_streaming))

    def test_delete_conversation_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(
            **{'message': 'successfully deleted the conversation'})
        conversations_object = Conversation("6549352073920512")

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.delete_conversation", Mock(return_value=demo_response)):
            self.assertEqual(
                demo_response, conversations_object.delete_conversation())

    def test_get_formatted_transcript_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(
            **{'transcript': {'content_type': 'text/markdown', 'payload': 'Natalia: Let us talk about India Australia'}})
        conversations_object = Conversation("6549352073920512")

        payload = {
            'contentType': 'text/markdown',
            # 'contentType': 'text/srt',
            'createParagraphs': "true",
            'phrases': {
                'highlightOnlyInsightKeyPhrases': "true",
                'highlightAllKeyPhrases': "true"
            },
            'showSpeakerSeparation': "true"
        }
        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.get_formatted_transcript", Mock(return_value=demo_response)):
            self.assertEqual(
                demo_response, conversations_object.get_formatted_transcript( parameters=payload))


if __name__ == '__main__':
    unittest.main()



from symbl.Conversations import Conversation
from symbl.utils.Logger import Log
from symbl.utils.Decorators import wrap_keyboard_interrupt
from symbl.utils.Threads import Thread
from time import sleep
import json
import websocket


class StreamingConnection():

    def __init__(self, url: str, connectionId: str, start_request: dict):
        self.conversation = Conversation(None)
        self.connectionId = connectionId
        self.url = url
        self.event_callbacks = {}
        self.start_request = start_request
        self.connection = None
        self.__connect()


    def __connect(self):
        if self.connection == None:

            self.connection = websocket.WebSocketApp(url=self.url, on_message=lambda this, data: self.__listen_to_events(data), on_error=lambda error: Log.getInstance().error(error))

            Thread.getInstance().start_on_thread(target=self.connection.run_forever)
            conn_timeout = 5
            while not self.connection.sock.connected and conn_timeout:
                sleep(1)
                conn_timeout -= 1

            self.connection.send(json.dumps(self.start_request))
    
    def __set_conversation(self, conversationId: str):
        self.conversation = Conversation(conversationId)

    def __listen_to_events(self, data):
        try:
            decoded_data = data if type(data) == str else data.decode('utf-8')
            json_data = json.loads(decoded_data)
            if 'type' in json_data and json_data['type'] == 'message' and 'type' in json_data['message'] and json_data['message']['type'] == "conversation_created":
                self.__set_conversation(str(json_data['message']['data']['conversationId']))
                Log.getInstance().info("Conversation id is {}".format(str(json_data['message']['data']['conversationId'])))
            elif 'type' in json_data and json_data['type'] == 'message' and 'type' in json_data['message'] and json_data['message']['type'] == "started_listening":
                Log.getInstance().info("Started Listening...")
            elif 'type' in json_data and json_data['type'] in self.event_callbacks:
                self.event_callbacks[json_data['type']](json_data) 
        except Exception as error:
            Log.getInstance().error(error)
            
    def subscribe(self, event_callbacks: dict):
        self.event_callbacks = event_callbacks
        
    def stop(self):
        if self.connection != None:
            stop_payload = {'type': 'stop_request'}
            self.connection.send(str(stop_payload))
    
    @wrap_keyboard_interrupt
    def send_audio(self, data):
        if self.connection != None:
            self.connection.send(data, opcode=websocket.ABNF.OPCODE_BINARY)

    @wrap_keyboard_interrupt
    def send_audio_from_mic(self, device=None):
        import pyaudio
        audio = pyaudio.PyAudio()
        chunk = 4096
        samplerate = self.start_request['config']['speechRecognition']['sampleRateHertz']
        stream = audio.open(format=pyaudio.paInt16, channels=1, rate=samplerate, input=True, frames_per_buffer=chunk)
        while True:
            data = stream.read(chunk)
            self.send_audio(data)



from symbl.jobs_api.Job import Job
from symbl.conversations_api.ConversationsApi import ConversationsApi
from symbl.utils.Logger import Log


def validate_conversation_id(function):
    def wrapper(*args, **kwargs):
        self = args[0]
        if self.get_conversation_id() != None:
            return function(*args, **kwargs)
        else:
            Log.getInstance().error("Conversation not initialized")
    return wrapper
class Conversation():

    __INTERVAL_TIME_IN_SECONDS = 30  ## in seconds

    def __init__(self, conversation_id: str, job_id: str=None,  wait=True, credentials=None):

        self.__conversation_id = conversation_id
        self.__wait = wait
        self.__credentials = credentials
        self.__conversation_api = ConversationsApi()

        if job_id != None:
            self.__job = Job(conversation_id=conversation_id, job_id=job_id, wait=wait)
            self.__job.monitor_job(self, interval=self.__INTERVAL_TIME_IN_SECONDS, wait=self.__wait, credentials=self.__credentials)
        else:
            self.__job = None
    
    def get_conversation_id(self):
        return self.__conversation_id
    
    def get_job_status(self):
        if self.__job != None:
            return self.__job.get_job_status()
        return None

    def get_job_id(self):
        if self.__job != None:
            return self.__job.get_job_id()
        return None

    def on_complete(self, func):
        self.__job.on_complete(func)
        return self

    def on_error(self, func):
        self.__job.on_error(func)
        return self

    @validate_conversation_id
    def get_action_items(self):
        return self.__conversation_api.get_action_items(self.__conversation_id, credentials=self.__credentials)

    @validate_conversation_id
    def get_follow_ups(self):
        return self.__conversation_api.get_follow_ups(self.__conversation_id, credentials=self.__credentials)
  
    @validate_conversation_id
    def get_members(self):  
        return self.__conversation_api.get_members(self.__conversation_id, credentials=self.__credentials)
  
    @validate_conversation_id
    def get_messages(self, parameters={}):
        return self.__conversation_api.get_messages(self.__conversation_id, credentials=self.__credentials, parameters=parameters)
  
    @validate_conversation_id
    def get_questions(self):  
        return self.__conversation_api.get_questions(self.__conversation_id, credentials=self.__credentials)
  
    @validate_conversation_id
    def get_topics(self, parameters={}):
        return self.__conversation_api.get_topics(self.__conversation_id, credentials=self.__credentials, parameters=parameters)

    @validate_conversation_id
    def get_conversation(self):
        return self.__conversation_api.get_conversation(self.__conversation_id, credentials=self.__credentials)

    @validate_conversation_id
    def get_trackers(self):  
        return self.__conversation_api.get_trackers(self.__conversation_id, credentials=self.__credentials)
  
    @validate_conversation_id
    def get_entities(self):  
        return self.__conversation_api.get_entities(self.__conversation_id, credentials=self.__credentials)
  
    @validate_conversation_id
    def get_analytics(self):  
        return self.__conversation_api.get_analytics(self.__conversation_id, credentials=self.__credentials)
  
    @validate_conversation_id
    def put_members(self, members_id, parameters={}):
        return self.__conversation_api.put_members(self.__conversation_id, members_id, parameters, credentials=self.__credentials)

    @validate_conversation_id
    def put_speakers_events(self, parameters={}):
        return self.__conversation_api.put_speakers_events(self.__conversation_id, parameters, credentials=self.__credentials)

    @validate_conversation_id
    def delete_conversation(self):  
        return self.__conversation_api.delete_conversation(self.__conversation_id, credentials=self.__credentials)
  
    @validate_conversation_id
    def get_formatted_transcript(self, parameters={}):  
        return self.__conversation_api.get_formatted_transcript(self.__conversation_id,parameters,credentials=self.__credentials)

    @validate_conversation_id
    def get_call_score(self):
        return self.__conversation_api.get_call_score(self.__conversation_id, credentials=self.__credentials)

    @validate_conversation_id
    def get_call_score_status(self):
        return self.__conversation_api.get_call_score_status(self.__conversation_id, credentials=self.__credentials)
  


from conversations_api.ConversationsApi import ConversationsApi

conversations = ConversationsApi();

conversation_id=6750867925630976 # Update with the conversation Id of your conversation

print(conversations.get_call_score_insights_url(conversation_id))
